@@ -24,6 +24,7 @@ import org.mozilla.javascript.ContextFactory;
 import org.mozilla.javascript.ImporterTopLevel;
 import org.mozilla.javascript.Kit;
 import org.mozilla.javascript.NativeCall;
+import org.mozilla.javascript.NativeObject;
 import org.mozilla.javascript.ScriptRuntime;
 import org.mozilla.javascript.Scriptable;
 import org.mozilla.javascript.ScriptableObject;
@@ -542,9 +543,9 @@ public class Dim {
             String name = (String) id;
             if (name.equals("this")) {
                 result = scriptable;
-            } else if (name.equals("__proto__")) {
+            } else if (name.equals(NativeObject.PROTO_PROPERTY)) {
                 result = scriptable.getPrototype();
-            } else if (name.equals("__parent__")) {
+            } else if (name.equals(NativeObject.PARENT_PROPERTY)) {
                 result = scriptable.getParentScope();
             } else {
                 result = ScriptableObject.getProperty(scriptable, name);
@@ -565,7 +566,7 @@ public class Dim {
     /** Returns an array of the property names on the given script object. */
     private Object[] getObjectIdsImpl(Context cx, Object object) {
         if (!(object instanceof Scriptable) || object == Undefined.instance) {
-            return Context.emptyArgs;
+            return ScriptRuntime.emptyArgs;
         }
 
         Object[] ids;
@@ -591,10 +592,10 @@ public class Dim {
             ids = tmp;
             extra = 0;
             if (proto != null) {
-                ids[extra++] = "__proto__";
+                ids[extra++] = NativeObject.PROTO_PROPERTY;
             }
             if (parent != null) {
-                ids[extra++] = "__parent__";
+                ids[extra++] = NativeObject.PARENT_PROPERTY;
             }
         }
 
