@@ -39,6 +39,7 @@ import org.mozilla.javascript.ScriptableObject;
 import org.mozilla.javascript.SecurityController;
 import org.mozilla.javascript.commonjs.module.ModuleScope;
 import org.mozilla.javascript.commonjs.module.Require;
+import org.mozilla.javascript.config.RhinoConfig;
 import org.mozilla.javascript.tools.SourceReader;
 import org.mozilla.javascript.tools.ToolErrorReporter;
 
@@ -132,7 +133,7 @@ public class Main {
      */
     public static void main(String args[]) {
         try {
-            if (Boolean.getBoolean("rhino.use_java_policy_security")) {
+            if (RhinoConfig.get("rhino.use_java_policy_security", false)) {
                 initJavaPolicySecuritySupport();
             }
         } catch (SecurityException ex) {
@@ -200,7 +201,8 @@ public class Main {
         try {
             Script script = cx.compileString(scriptText, "<command>", 1, null);
             if (script != null) {
-                script.exec(cx, getShellScope());
+                Scriptable scope = getShellScope();
+                script.exec(cx, scope, scope);
             }
         } catch (RhinoException rex) {
             ToolErrorReporter.reportException(cx.getErrorReporter(), rex);
@@ -480,7 +482,7 @@ public class Main {
                     String finalSource = source.toString();
                     Script script = cx.compileString(finalSource, "<stdin>", lineno, null);
                     if (script != null) {
-                        Object result = script.exec(cx, scope);
+                        Object result = script.exec(cx, scope, scope);
                         // Avoid printing out undefined or function definitions.
                         if (result != Context.getUndefinedValue()
                                 && !(result instanceof Function
@@ -579,7 +581,7 @@ public class Main {
         }
 
         if (script != null) {
-            script.exec(cx, scope);
+            script.exec(cx, scope, scope);
         }
     }
 
