@@ -15,6 +15,8 @@ import java.lang.reflect.Member;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import org.mozilla.javascript.commonjs.module.ModuleScope;
+import org.mozilla.javascript.lc.type.TypeInfo;
+import org.mozilla.javascript.lc.type.TypeInfoFactory;
 
 public class FunctionObject extends BaseFunction {
     private static final long serialVersionUID = -5332312783643935019L;
@@ -22,7 +24,7 @@ public class FunctionObject extends BaseFunction {
     /**
      * Create a JavaScript function object from a Java method.
      *
-     * <p>The <code>member</code> argument must be either a java.lang.reflect.Method or a
+     * <p>The {@code member} argument must be either a java.lang.reflect.Method or a
      * java.lang.reflect.Constructor and must match one of two forms.
      *
      * <p>The first form is a member with zero or more parameters of the following types: Object,
@@ -59,19 +61,17 @@ public class FunctionObject extends BaseFunction {
      *
      * and an Object result.
      *
-     * <p>When the function varargs form is called as part of a function call, the <code>args</code>
-     * parameter contains the arguments, with <code>thisObj</code> set to the JavaScript 'this'
-     * value. <code>funObj</code> is the function object for the invoked function.
+     * <p>When the function varargs form is called as part of a function call, the {@code args}
+     * parameter contains the arguments, with {@code thisObj} set to the JavaScript 'this' value.
+     * {@code funObj} is the function object for the invoked function.
      *
-     * <p>When the constructor varargs form is called or invoked while evaluating a <code>new</code>
-     * expression, <code>args</code> contains the arguments, <code>ctorObj</code> refers to this
-     * FunctionObject, and <code>inNewExpr</code> is true if and only if a <code>new</code>
-     * expression caused the call. This supports defining a function that has different behavior
-     * when called as a constructor than when invoked as a normal function call. (For example, the
-     * Boolean constructor, when called as a function, will convert to boolean rather than creating
-     * a new object.)
-     *
-     * <p>
+     * <p>When the constructor varargs form is called or invoked while evaluating a {@code new}
+     * expression, {@code args} contains the arguments, {@code ctorObj} refers to this
+     * FunctionObject, and {@code inNewExpr} is true if and only if a {@code new} expression caused
+     * the call. This supports defining a function that has different behavior when called as a
+     * constructor than when invoked as a normal function call. (For example, the Boolean
+     * constructor, when called as a function, will convert to boolean rather than creating a new
+     * object.)
      *
      * @param name the name of the function
      * @param methodOrConstructor a java.lang.reflect.Method or a java.lang.reflect.Constructor that
@@ -80,34 +80,38 @@ public class FunctionObject extends BaseFunction {
      * @see org.mozilla.javascript.Scriptable
      */
     public FunctionObject(String name, Member methodOrConstructor, Scriptable scope) {
+        // fallback to global factory for compatibility with old behaviour, where the `scope` can be
+        // an object not yet initialized via `initStandardObject(...)`
+        var typeInfoFactory = TypeInfoFactory.getOrElse(scope, TypeInfoFactory.GLOBAL);
+
         if (methodOrConstructor instanceof Constructor) {
-            member = new MemberBox((Constructor<?>) methodOrConstructor);
+            member = new MemberBox((Constructor<?>) methodOrConstructor, typeInfoFactory);
             isStatic = true; // well, doesn't take a 'this'
         } else {
-            member = new MemberBox((Method) methodOrConstructor);
+            member = new MemberBox((Method) methodOrConstructor, typeInfoFactory);
             isStatic = member.isStatic();
         }
         String methodName = member.getName();
         this.functionName = name;
-        Class<?>[] types = member.argTypes;
-        int arity = types.length;
-        if (arity == 4 && (types[1].isArray() || types[2].isArray())) {
+        var types = member.getArgTypes();
+        int arity = types.size();
+        if (arity == 4 && (types.get(1).isArray() || types.get(2).isArray())) {
             // Either variable args or an error.
-            if (types[1].isArray()) {
+            if (types.get(1).isArray()) {
                 if (!isStatic
-                        || types[0] != ScriptRuntime.ContextClass
-                        || types[1].getComponentType() != ScriptRuntime.ObjectClass
-                        || types[2] != ScriptRuntime.FunctionClass
-                        || types[3] != Boolean.TYPE) {
+                        || types.get(0).isNot(Context.class)
+                        || types.get(1).isNot(Object[].class)
+                        || types.get(2).isNot(Function.class)
+                        || types.get(3).isNot(boolean.class)) {
                     throw Context.reportRuntimeErrorById("msg.varargs.ctor", methodName);
                 }
                 parmsLength = VARARGS_CTOR;
             } else {
                 if (!isStatic
-                        || types[0] != ScriptRuntime.ContextClass
-                        || types[1] != ScriptRuntime.ScriptableClass
-                        || types[2].getComponentType() != ScriptRuntime.ObjectClass
-                        || types[3] != ScriptRuntime.FunctionClass) {
+                        || types.get(0).isNot(Context.class)
+                        || types.get(1).isNot(Scriptable.class)
+                        || types.get(2).isNot(Object[].class)
+                        || types.get(3).isNot(Function.class)) {
                     throw Context.reportRuntimeErrorById("msg.varargs.fun", methodName);
                 }
                 parmsLength = VARARGS_METHOD;
@@ -117,10 +121,10 @@ public class FunctionObject extends BaseFunction {
             if (arity > 0) {
                 typeTags = new byte[arity];
                 for (int i = 0; i != arity; ++i) {
-                    int tag = getTypeTag(types[i]);
+                    int tag = types.get(i).getTypeTag();
                     if (tag == JAVA_UNSUPPORTED_TYPE) {
                         throw Context.reportRuntimeErrorById(
-                                "msg.bad.parms", types[i].getName(), methodName);
+                                "msg.bad.parms", types.get(i).asClass().getName(), methodName);
                     }
                     typeTags[i] = (byte) tag;
                 }
@@ -146,8 +150,9 @@ public class FunctionObject extends BaseFunction {
     }
 
     /**
-     * @return One of <code>JAVA_*_TYPE</code> constants to indicate desired type or {@link
+     * @return One of {@code JAVA_*_TYPE} constants to indicate desired type or {@link
      *     #JAVA_UNSUPPORTED_TYPE} if the conversion is not possible
+     * @see TypeInfo#getTypeTag()
      */
     public static int getTypeTag(Class<?> type) {
         if (type == ScriptRuntime.StringClass) return JAVA_STRING_TYPE;
@@ -164,19 +169,32 @@ public class FunctionObject extends BaseFunction {
     }
 
     public static Object convertArg(Context cx, Scriptable scope, Object arg, int typeTag) {
+        return convertArg(cx, scope, arg, typeTag, false);
+    }
+
+    public static Object convertArg(
+            Context cx, Scriptable scope, Object arg, int typeTag, boolean isNullable) {
         switch (typeTag) {
             case JAVA_STRING_TYPE:
                 if (arg instanceof String) return arg;
-                return ScriptRuntime.toString(arg);
+                return (arg == null && isNullable) ? null : ScriptRuntime.toString(arg);
             case JAVA_INT_TYPE:
                 if (arg instanceof Integer) return arg;
-                return Integer.valueOf(ScriptRuntime.toInt32(arg));
+                return (arg == null && isNullable)
+                        ? null
+                        : Integer.valueOf(ScriptRuntime.toInt32(arg));
             case JAVA_BOOLEAN_TYPE:
                 if (arg instanceof Boolean) return arg;
-                return ScriptRuntime.toBoolean(arg) ? Boolean.TRUE : Boolean.FALSE;
+                if (arg == null && isNullable) {
+                    return null;
+                } else {
+                    return ScriptRuntime.toBoolean(arg) ? Boolean.TRUE : Boolean.FALSE;
+                }
             case JAVA_DOUBLE_TYPE:
                 if (arg instanceof Double) return arg;
-                return Double.valueOf(ScriptRuntime.toNumber(arg));
+                return (arg == null && isNullable)
+                        ? null
+                        : Double.valueOf(ScriptRuntime.toNumber(arg));
             case JAVA_SCRIPTABLE_TYPE:
                 return ScriptRuntime.toObjectOrNull(cx, arg, scope);
             case JAVA_OBJECT_TYPE:
@@ -274,7 +292,7 @@ public class FunctionObject extends BaseFunction {
      *
      * <p>Sets up the "prototype" and "constructor" properties. Also calls setParent and
      * setPrototype with appropriate values. Then adds the function object as a property of the
-     * given scope, using <code>prototype.getClassName()</code> as the name of the property.
+     * given scope, using {@code prototype.getClassName()} as the name of the property.
      *
      * @param scope the scope in which to define the constructor (typically the global object)
      * @param prototype the prototype object
@@ -295,7 +313,7 @@ public class FunctionObject extends BaseFunction {
      *
      * <p>Sets up the "prototype" and "constructor" properties. Also calls setParent and
      * setPrototype with appropriate values. Then adds the function object as a property of the
-     * given scope, using <code>prototype.getClassName()</code> as the name of the property.
+     * given scope, using {@code prototype.getClassName()} as the name of the property.
      *
      * @param scope the scope in which to define the constructor (typically the global object)
      * @param prototype the prototype object
@@ -321,7 +339,7 @@ public class FunctionObject extends BaseFunction {
 
     /**
      * @deprecated Use {@link #getTypeTag(Class)} and {@link #convertArg(Context, Scriptable,
-     *     Object, int)} for type conversion.
+     *     Object, int, boolean)} for type conversion.
      */
     @Deprecated
     public static Object convertArg(Context cx, Scriptable scope, Object arg, Class<?> desired) {
@@ -329,7 +347,7 @@ public class FunctionObject extends BaseFunction {
         if (tag == JAVA_UNSUPPORTED_TYPE) {
             throw Context.reportRuntimeErrorById("msg.cant.convert", desired.getName());
         }
-        return convertArg(cx, scope, arg, tag);
+        return convertArg(cx, scope, arg, tag, false);
     }
 
     /**
@@ -377,7 +395,7 @@ public class FunctionObject extends BaseFunction {
                 if (!clazz.isInstance(thisObj)) {
                     boolean compatible = false;
                     if (thisObj == scope || thisObj instanceof ModuleScope) {
-                        Scriptable parentScope = getParentScope();
+                        Scriptable parentScope = getDeclarationScope();
                         if (scope != parentScope) {
                             // Call with dynamic scope for standalone function,
                             // use parentScope as thisObj
@@ -401,7 +419,13 @@ public class FunctionObject extends BaseFunction {
                 invokeArgs = args;
                 for (int i = 0; i != parmsLength; ++i) {
                     Object arg = args[i];
-                    Object converted = convertArg(cx, scope, arg, typeTags[i]);
+                    Object converted =
+                            convertArg(
+                                    cx,
+                                    scope,
+                                    arg,
+                                    typeTags[i],
+                                    member.getArgNullability().isNullable(i));
                     if (arg != converted) {
                         if (invokeArgs == args) {
                             invokeArgs = args.clone();
@@ -415,7 +439,13 @@ public class FunctionObject extends BaseFunction {
                 invokeArgs = new Object[parmsLength];
                 for (int i = 0; i != parmsLength; ++i) {
                     Object arg = (i < argsLength) ? args[i] : Undefined.instance;
-                    invokeArgs[i] = convertArg(cx, scope, arg, typeTags[i]);
+                    invokeArgs[i] =
+                            convertArg(
+                                    cx,
+                                    scope,
+                                    arg,
+                                    typeTags[i],
+                                    member.getArgNullability().isNullable(i));
                 }
             }
 
@@ -431,7 +461,7 @@ public class FunctionObject extends BaseFunction {
             if (hasVoidReturn) {
                 result = Undefined.instance;
             } else if (returnTypeTag == JAVA_UNSUPPORTED_TYPE) {
-                result = cx.getWrapFactory().wrap(cx, scope, result, null);
+                result = cx.getWrapFactory().wrap(cx, scope, result, TypeInfo.NONE);
             }
             // XXX: the code assumes that if returnTypeTag == JAVA_OBJECT_TYPE
             // then the Java method did a proper job of converting the
@@ -460,7 +490,7 @@ public class FunctionObject extends BaseFunction {
         }
 
         result.setPrototype(getClassPrototype());
-        result.setParentScope(getParentScope());
+        result.setParentScope(getDeclarationScope());
         return result;
     }
 
@@ -475,10 +505,10 @@ public class FunctionObject extends BaseFunction {
     private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
         in.defaultReadObject();
         if (parmsLength > 0) {
-            Class<?>[] types = member.argTypes;
+            var types = member.getArgTypes();
             typeTags = new byte[parmsLength];
             for (int i = 0; i != parmsLength; ++i) {
-                typeTags[i] = (byte) getTypeTag(types[i]);
+                typeTags[i] = (byte) types.get(i).getTypeTag();
             }
         }
         if (member.isMethod()) {
