@@ -6,11 +6,15 @@
 
 package org.mozilla.javascript;
 
-import java.lang.reflect.Array;
 import java.lang.reflect.Method;
 import java.util.Arrays;
 import java.util.EnumSet;
+import java.util.Map;
 import java.util.concurrent.CopyOnWriteArrayList;
+import org.mozilla.javascript.lc.type.ParameterizedTypeInfo;
+import org.mozilla.javascript.lc.type.TypeInfo;
+import org.mozilla.javascript.lc.type.TypeInfoFactory;
+import org.mozilla.javascript.lc.type.VariableTypeInfo;
 
 /**
  * This class reflects Java methods into the JavaScript environment and handles overloading of
@@ -40,8 +44,9 @@ public class NativeJavaMethod extends BaseFunction {
         this.methods = new MemberBox[] {method};
     }
 
+    @Deprecated
     public NativeJavaMethod(Method method, String name) {
-        this(new MemberBox(method), name);
+        this(new MemberBox(method, TypeInfoFactory.GLOBAL), name);
     }
 
     @Override
@@ -114,7 +119,7 @@ public class NativeJavaMethod extends BaseFunction {
             } else {
                 sb.append(methods[i].getName());
             }
-            sb.append(JavaMembers.liveConnectSignature(methods[i].argTypes));
+            sb.append(JavaMembers.liveConnectSignature(methods[i].getArgTypes()));
             sb.append('\n');
         }
         return sb.toString();
@@ -135,53 +140,18 @@ public class NativeJavaMethod extends BaseFunction {
         }
 
         MemberBox meth = methods[index];
-        Class<?>[] argTypes = meth.argTypes;
 
-        if (meth.vararg) {
-            // marshall the explicit parameters
-            Object[] newArgs = new Object[argTypes.length];
-            for (int i = 0; i < argTypes.length - 1; i++) {
-                newArgs[i] = Context.jsToJava(args[i], argTypes[i]);
-            }
-
-            Object varArgs;
-
-            // Handle special situation where a single variable parameter
-            // is given and it is a Java or ECMA array or is null.
-            if (args.length == argTypes.length
-                    && (args[args.length - 1] == null
-                            || args[args.length - 1] instanceof NativeArray
-                            || args[args.length - 1] instanceof NativeJavaArray)) {
-                // convert the ECMA array into a native array
-                varArgs = Context.jsToJava(args[args.length - 1], argTypes[argTypes.length - 1]);
-            } else {
-                // marshall the variable parameters
-                Class<?> componentType = argTypes[argTypes.length - 1].getComponentType();
-                varArgs = Array.newInstance(componentType, args.length - argTypes.length + 1);
-                for (int i = 0; i < Array.getLength(varArgs); i++) {
-                    Object value = Context.jsToJava(args[argTypes.length - 1 + i], componentType);
-                    Array.set(varArgs, i, value);
-                }
-            }
-
-            // add varargs
-            newArgs[argTypes.length - 1] = varArgs;
-            // replace the original args with the new one
-            args = newArgs;
-        } else {
-            // First, we marshall the args.
-            Object[] origArgs = args;
-            for (int i = 0; i < args.length; i++) {
-                Object arg = args[i];
-                Object coerced = Context.jsToJava(arg, argTypes[i]);
-                if (coerced != arg) {
-                    if (origArgs == args) {
-                        args = args.clone();
-                    }
-                    args[i] = coerced;
-                }
+        Map<VariableTypeInfo, TypeInfo> mapping = Map.of();
+        if (thisObj instanceof NativeJavaObject) {
+            var staticType = ((NativeJavaObject) thisObj).staticType;
+            if (staticType instanceof ParameterizedTypeInfo) {
+                mapping =
+                        ((ParameterizedTypeInfo) staticType)
+                                .extractConsolidationMapping(TypeInfoFactory.get(scope));
             }
         }
+        args = meth.wrapArgsInternal(args, mapping);
+
         Object javaObject;
         if (meth.isStatic()) {
             javaObject = null; // don't need an object
@@ -209,33 +179,35 @@ public class NativeJavaMethod extends BaseFunction {
             printDebug("Calling ", meth, args);
         }
 
-        Object retval = meth.invoke(javaObject, args);
-        Class<?> staticType = meth.method().getReturnType();
+        var returnValue = meth.invoke(javaObject, args);
+        var returnType = meth.getReturnType();
 
         if (debug) {
-            Class<?> actualType = (retval == null) ? null : retval.getClass();
+            Class<?> actualType = (returnValue == null) ? null : returnValue.getClass();
             System.err.println(
                     " ----- Returned "
-                            + retval
+                            + returnValue
                             + " actual = "
                             + actualType
                             + " expect = "
-                            + staticType);
+                            + returnType.asClass());
+        }
+
+        if (returnType == TypeInfo.PRIMITIVE_VOID) {
+            // skip result wrapping if we don't need result at all
+            return Undefined.instance;
         }
 
         Object wrapped =
                 cx.getWrapFactory()
                         .wrap(
                                 cx, scope,
-                                retval, staticType);
+                                returnValue, returnType);
         if (debug) {
             Class<?> actualType = (wrapped == null) ? null : wrapped.getClass();
             System.err.println(" ----- Wrapped as " + wrapped + " class = " + actualType);
         }
 
-        if (wrapped == null && staticType == Void.TYPE) {
-            wrapped = Undefined.instance;
-        }
         return wrapped;
     }
 
@@ -265,144 +237,115 @@ public class NativeJavaMethod extends BaseFunction {
     static int findFunction(Context cx, MemberBox[] methodsOrCtors, Object[] args) {
         if (methodsOrCtors.length == 0) {
             return -1;
-        } else if (methodsOrCtors.length == 1) {
-            MemberBox member = methodsOrCtors[0];
-            Class<?>[] argTypes = member.argTypes;
-            int alength = argTypes.length;
-
-            if (member.vararg) {
-                alength--;
-                if (alength > args.length) {
-                    return -1;
-                }
-            } else {
-                if (alength != args.length) {
-                    return -1;
-                }
+        }
+        if (methodsOrCtors.length == 1) {
+            if (failFastConversionWeights(args, methodsOrCtors[0]) == null) {
+                return -1;
             }
-            for (int j = 0; j != alength; ++j) {
-                if (!NativeJavaObject.canConvert(args[j], argTypes[j])) {
-                    if (debug) printDebug("Rejecting (args can't convert) ", member, args);
-                    return -1;
-                }
-            }
-            if (debug) printDebug("Found ", member, args);
+            if (debug) printDebug("Found ", methodsOrCtors[0], args);
             return 0;
         }
 
         int firstBestFit = -1;
+        int[] firstBestFitWeights = null;
+
         int[] extraBestFits = null;
+        int[][] extraBestFitWeights = null;
         int extraBestFitsCount = 0;
 
         search:
         for (int i = 0; i < methodsOrCtors.length; i++) {
             MemberBox member = methodsOrCtors[i];
-            Class<?>[] argTypes = member.argTypes;
-            int alength = argTypes.length;
-            if (member.vararg) {
-                alength--;
-                if (alength > args.length) {
-                    continue search;
-                }
-            } else {
-                if (alength != args.length) {
-                    continue search;
-                }
-            }
-            for (int j = 0; j < alength; j++) {
-                if (!NativeJavaObject.canConvert(args[j], argTypes[j])) {
-                    if (debug) printDebug("Rejecting (args can't convert) ", member, args);
-                    continue search;
-                }
+
+            final var weights = failFastConversionWeights(args, member);
+            if (weights == null) {
+                continue search;
             }
+
             if (firstBestFit < 0) {
                 if (debug) printDebug("Found first applicable ", member, args);
                 firstBestFit = i;
-            } else {
-                // Compare with all currently fit methods.
-                // The loop starts from -1 denoting firstBestFit and proceed
-                // until extraBestFitsCount to avoid extraBestFits allocation
-                // in the most common case of no ambiguity
-                int betterCount = 0; // number of times member was preferred over
-                // best fits
-                int worseCount = 0; // number of times best fits were preferred
-                // over member
-                for (int j = -1; j != extraBestFitsCount; ++j) {
-                    int bestFitIndex;
-                    if (j == -1) {
-                        bestFitIndex = firstBestFit;
+                firstBestFitWeights = weights;
+                continue search;
+            }
+
+            // Compare with all currently fit methods.
+            // The loop starts from -1 denoting firstBestFit and proceed
+            // until extraBestFitsCount to avoid extraBestFits allocation
+            // in the most common case of no ambiguity
+            int betterCount = 0; // number of times member was preferred over
+            // best fits
+            int worseCount = 0; // number of times best fits were preferred
+            // over member
+            for (int j = -1; j != extraBestFitsCount; ++j) {
+                int bestFitIndex = j < 0 ? firstBestFit : extraBestFits[j];
+                MemberBox bestFit = methodsOrCtors[bestFitIndex];
+                int[] bestFitWeights = j < 0 ? firstBestFitWeights : extraBestFitWeights[j];
+                if (cx.hasFeature(Context.FEATURE_ENHANCED_JAVA_ACCESS)
+                        && bestFit.isPublic() != member.isPublic()) {
+                    // When FEATURE_ENHANCED_JAVA_ACCESS gives us access
+                    // to non-public members, continue to prefer public
+                    // methods in overloading
+                    if (!bestFit.isPublic()) ++betterCount;
+                    else ++worseCount;
+                } else {
+                    int preference =
+                            preferSignature(args, member, weights, bestFit, bestFitWeights);
+                    if (preference == PREFERENCE_AMBIGUOUS) {
+                        break;
+                    } else if (preference == PREFERENCE_FIRST_ARG) {
+                        ++betterCount;
+                    } else if (preference == PREFERENCE_SECOND_ARG) {
+                        ++worseCount;
                     } else {
-                        bestFitIndex = extraBestFits[j];
-                    }
-                    MemberBox bestFit = methodsOrCtors[bestFitIndex];
-                    if (cx.hasFeature(Context.FEATURE_ENHANCED_JAVA_ACCESS)
-                            && bestFit.isPublic() != member.isPublic()) {
-                        // When FEATURE_ENHANCED_JAVA_ACCESS gives us access
-                        // to non-public members, continue to prefer public
-                        // methods in overloading
-                        if (!bestFit.isPublic()) ++betterCount;
-                        else ++worseCount;
-                    } else {
-                        int preference =
-                                preferSignature(
-                                        args,
-                                        argTypes,
-                                        member.vararg,
-                                        bestFit.argTypes,
-                                        bestFit.vararg);
-                        if (preference == PREFERENCE_AMBIGUOUS) {
-                            break;
-                        } else if (preference == PREFERENCE_FIRST_ARG) {
-                            ++betterCount;
-                        } else if (preference == PREFERENCE_SECOND_ARG) {
-                            ++worseCount;
-                        } else {
-                            if (preference != PREFERENCE_EQUAL) Kit.codeBug();
-                            // This should not happen in theory
-                            // but on some JVMs, Class.getMethods will return all
+                        if (preference != PREFERENCE_EQUAL) Kit.codeBug();
+                        // This should not happen in theory
+                        // but on some JVMs, Class.getMethods will return all
+                        // static methods of the class hierarchy, even if
+                        // a derived class's parameters match exactly.
+                        // We want to call the derived class's method.
+                        if (bestFit.isStatic()
+                                && bestFit.getDeclaringClass()
+                                        .isAssignableFrom(member.getDeclaringClass())) {
+                            // On some JVMs, Class.getMethods will return all
                             // static methods of the class hierarchy, even if
                             // a derived class's parameters match exactly.
                             // We want to call the derived class's method.
-                            if (bestFit.isStatic()
-                                    && bestFit.getDeclaringClass()
-                                            .isAssignableFrom(member.getDeclaringClass())) {
-                                // On some JVMs, Class.getMethods will return all
-                                // static methods of the class hierarchy, even if
-                                // a derived class's parameters match exactly.
-                                // We want to call the derived class's method.
-                                if (debug)
-                                    printDebug("Substituting (overridden static)", member, args);
-                                if (j == -1) {
-                                    firstBestFit = i;
-                                } else {
-                                    extraBestFits[j] = i;
-                                }
+                            if (debug) printDebug("Substituting (overridden static)", member, args);
+                            if (j == -1) {
+                                firstBestFit = i;
+                                firstBestFitWeights = weights;
                             } else {
-                                if (debug)
-                                    printDebug("Ignoring same signature member ", member, args);
+                                extraBestFits[j] = i;
+                                extraBestFitWeights[j] = weights;
                             }
-                            continue search;
+                        } else {
+                            if (debug) printDebug("Ignoring same signature member ", member, args);
                         }
+                        continue search;
                     }
                 }
-                if (betterCount == 1 + extraBestFitsCount) {
-                    // member was preferred over all best fits
-                    if (debug) printDebug("New first applicable ", member, args);
-                    firstBestFit = i;
-                    extraBestFitsCount = 0;
-                } else if (worseCount == 1 + extraBestFitsCount) {
-                    // all best fits were preferred over member, ignore it
-                    if (debug) printDebug("Rejecting (all current bests better) ", member, args);
-                } else {
-                    // some ambiguity was present, add member to best fit set
-                    if (debug) printDebug("Added to best fit set ", member, args);
-                    if (extraBestFits == null) {
-                        // Allocate maximum possible array
-                        extraBestFits = new int[methodsOrCtors.length - 1];
-                    }
-                    extraBestFits[extraBestFitsCount] = i;
-                    ++extraBestFitsCount;
+            }
+            if (betterCount == 1 + extraBestFitsCount) {
+                // member was preferred over all best fits
+                if (debug) printDebug("New first applicable ", member, args);
+                firstBestFit = i;
+                firstBestFitWeights = weights;
+                extraBestFitsCount = 0;
+            } else if (worseCount == 1 + extraBestFitsCount) {
+                // all best fits were preferred over member, ignore it
+                if (debug) printDebug("Rejecting (all current bests better) ", member, args);
+            } else {
+                // some ambiguity was present, add member to best fit set
+                if (debug) printDebug("Added to best fit set ", member, args);
+                if (extraBestFits == null) {
+                    // Allocate maximum possible array
+                    extraBestFits = new int[methodsOrCtors.length - 1];
+                    extraBestFitWeights = new int[methodsOrCtors.length - 1][];
                 }
+                extraBestFits[extraBestFitsCount] = i;
+                extraBestFitWeights[extraBestFitsCount] = weights;
+                ++extraBestFitsCount;
             }
         }
 
@@ -453,24 +396,44 @@ public class NativeJavaMethod extends BaseFunction {
     private static final int PREFERENCE_AMBIGUOUS = 3;
 
     /**
-     * Determine which of two signatures is the closer fit. Returns one of PREFERENCE_EQUAL,
-     * PREFERENCE_FIRST_ARG, PREFERENCE_SECOND_ARG, or PREFERENCE_AMBIGUOUS.
+     * Determine which of two signatures is the closer fit. Returns one of {@link
+     * #PREFERENCE_EQUAL}, {@link #PREFERENCE_FIRST_ARG}, {@link #PREFERENCE_SECOND_ARG}, or {@link
+     * #PREFERENCE_AMBIGUOUS}.
      */
     private static int preferSignature(
-            Object[] args, Class<?>[] sig1, boolean vararg1, Class<?>[] sig2, boolean vararg2) {
+            Object[] args,
+            MemberBox member1,
+            int[] computedWeights1,
+            MemberBox member2,
+            int[] computedWeights2) {
+        final var types1 = member1.getArgTypes();
+        final var types2 = member2.getArgTypes();
+
         int totalPreference = 0;
         for (int j = 0; j < args.length; j++) {
-            Class<?> type1 = vararg1 && j >= sig1.length ? sig1[sig1.length - 1] : sig1[j];
-            Class<?> type2 = vararg2 && j >= sig2.length ? sig2[sig2.length - 1] : sig2[j];
-            if (type1 == type2) {
+            final var type1 =
+                    member1.vararg && j >= types1.size()
+                            ? types1.get(types1.size() - 1)
+                            : types1.get(j);
+            final var type2 =
+                    member2.vararg && j >= types2.size()
+                            ? types2.get(types2.size() - 1)
+                            : types2.get(j);
+            if (type1.asClass() == type2.asClass()) {
                 continue;
             }
-            Object arg = args[j];
+            final var arg = args[j];
 
             // Determine which of type1, type2 is easier to convert from arg.
 
-            int rank1 = NativeJavaObject.getConversionWeight(arg, type1);
-            int rank2 = NativeJavaObject.getConversionWeight(arg, type2);
+            final var rank1 =
+                    j < computedWeights1.length
+                            ? computedWeights1[j]
+                            : NativeJavaObject.getConversionWeight(arg, type1);
+            final var rank2 =
+                    j < computedWeights2.length
+                            ? computedWeights2[j]
+                            : NativeJavaObject.getConversionWeight(arg, type2);
 
             int preference;
             if (rank1 < rank2) {
@@ -501,6 +464,44 @@ public class NativeJavaMethod extends BaseFunction {
         return totalPreference;
     }
 
+    /**
+     * 1. {@code args} is too short for {@code member} calling -> return {@code null}
+     *
+     * <p>2. at least one arg cannot be converted -> return {@code null}
+     *
+     * <p>3. otherwise -> return an int array holding all computed conversion weights, whose length
+     * will be {@code args.length} for non-vararg member or {@code args.length-1} for vararg member
+     *
+     * @see NativeJavaObject#getConversionWeight(Object, org.mozilla.javascript.lc.type.TypeInfo)
+     * @see NativeJavaObject#canConvert(Object, org.mozilla.javascript.lc.type.TypeInfo)
+     */
+    static int[] failFastConversionWeights(Object[] args, MemberBox member) {
+        final var argTypes = member.getArgTypes();
+        var typeLen = argTypes.size();
+        if (member.vararg) {
+            typeLen--;
+            if (typeLen > args.length) {
+                return null;
+            }
+        } else {
+            if (typeLen != args.length) {
+                return null;
+            }
+        }
+        final var weights = new int[typeLen];
+        for (int i = 0; i < typeLen; i++) {
+            final var weight = NativeJavaObject.getConversionWeight(args[i], argTypes.get(i));
+            if (weight >= NativeJavaObject.CONVERSION_NONE) {
+                if (debug) {
+                    printDebug("Rejecting (args can't convert) ", member, args);
+                }
+                return null;
+            }
+            weights[i] = weight;
+        }
+        return weights;
+    }
+
     private static final boolean debug = false;
 
     private static void printDebug(String msg, MemberBox member, Object[] args) {
@@ -513,7 +514,7 @@ public class NativeJavaMethod extends BaseFunction {
             if (member.isMethod()) {
                 sb.append(member.getName());
             }
-            sb.append(JavaMembers.liveConnectSignature(member.argTypes));
+            sb.append(JavaMembers.liveConnectSignature(member.getArgTypes()));
             sb.append(" for arguments (");
             sb.append(scriptSignature(args));
             sb.append(')');
