@@ -18,14 +18,14 @@ import java.util.Date;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Objects;
+import org.mozilla.javascript.lc.type.TypeInfo;
+import org.mozilla.javascript.lc.type.TypeInfoFactory;
 
 /**
- * This class reflects non-Array Java objects into the JavaScript environment. It reflect fields
+ * This class reflects non-Array Java objects into the JavaScript environment. It reflects fields
  * directly, and uses NativeJavaMethod objects to reflect (possibly overloaded) methods. It also
  * provides iterator support for all iterable objects.
  *
- * <p>
- *
  * @author Mike Shaver
  * @see NativeJavaArray
  * @see NativeJavaPackage
@@ -41,12 +41,12 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
     public NativeJavaObject() {}
 
-    public NativeJavaObject(Scriptable scope, Object javaObject, Class<?> staticType) {
+    public NativeJavaObject(Scriptable scope, Object javaObject, TypeInfo staticType) {
         this(scope, javaObject, staticType, false);
     }
 
     public NativeJavaObject(
-            Scriptable scope, Object javaObject, Class<?> staticType, boolean isAdapter) {
+            Scriptable scope, Object javaObject, TypeInfo staticType, boolean isAdapter) {
         this.parent = scope;
         this.javaObject = javaObject;
         this.staticType = staticType;
@@ -59,9 +59,9 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
         if (javaObject != null) {
             dynamicType = javaObject.getClass();
         } else {
-            dynamicType = staticType;
+            dynamicType = staticType.asClass();
         }
-        members = JavaMembers.lookupClass(parent, dynamicType, staticType, isAdapter);
+        members = JavaMembers.lookupClass(parent, dynamicType, staticType.asClass(), isAdapter);
         fieldAndMethods = members.getFieldAndMethodsObjects(this, javaObject, false);
     }
 
@@ -234,7 +234,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 value =
                         f.call(
                                 Context.getContext(),
-                                f.getParentScope(),
+                                f.getDeclarationScope(),
                                 this,
                                 ScriptRuntime.emptyArgs);
             } else {
@@ -253,11 +253,16 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
      * Determine whether we can/should convert between the given type and the desired one. This
      * should be superceded by a conversion-cost calculation function, but for now I'll hide behind
      * precedent.
+     *
+     * @deprecated use {@link #canConvert(Object, TypeInfo)} instead
      */
+    @Deprecated
     public static boolean canConvert(Object fromObj, Class<?> to) {
-        int weight = getConversionWeight(fromObj, to);
+        return canConvert(fromObj, TypeInfoFactory.GLOBAL.create(to));
+    }
 
-        return (weight < CONVERSION_NONE);
+    public static boolean canConvert(Object fromObj, TypeInfo to) {
+        return getConversionWeight(fromObj, to) < CONVERSION_NONE;
     }
 
     private static final int JSTYPE_UNDEFINED = 0; // undefined type
@@ -282,12 +287,12 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
      * href="http://www.mozilla.org/js/liveconnect/lc3_method_overloading.html">"preferred method
      * conversions" from Live Connect 3</a>
      */
-    static int getConversionWeight(Object fromObj, Class<?> to) {
+    static int getConversionWeight(Object fromObj, TypeInfo to) {
         int fromCode = getJSTypeCode(fromObj);
 
         switch (fromCode) {
             case JSTYPE_UNDEFINED:
-                if (to == ScriptRuntime.StringClass || to == ScriptRuntime.ObjectClass) {
+                if (to == TypeInfo.STRING || to.isObjectExact()) {
                     return 1;
                 }
                 break;
@@ -300,13 +305,13 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
             case JSTYPE_BOOLEAN:
                 // "boolean" is #1
-                if (to == Boolean.TYPE) {
+                if (to == TypeInfo.PRIMITIVE_BOOLEAN) {
                     return 1;
-                } else if (to == ScriptRuntime.BooleanClass) {
+                } else if (to == TypeInfo.BOOLEAN) {
                     return 2;
-                } else if (to == ScriptRuntime.ObjectClass) {
+                } else if (to.isObjectExact()) {
                     return 3;
-                } else if (to == ScriptRuntime.StringClass) {
+                } else if (to == TypeInfo.STRING) {
                     return 4;
                 }
                 break;
@@ -314,20 +319,20 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
             case JSTYPE_NUMBER:
             case JSTYPE_BIGINT:
                 if (to.isPrimitive()) {
-                    if (to == Double.TYPE) {
+                    if (to.isDouble()) {
                         return 1;
-                    } else if (to != Boolean.TYPE) {
+                    } else if (!to.isBoolean()) {
                         return 1 + getSizeRank(to);
                     }
                 } else {
-                    if (to == ScriptRuntime.StringClass) {
+                    if (to == TypeInfo.STRING) {
                         // native numbers are #1-8
                         return 9;
-                    } else if (to == ScriptRuntime.BigIntegerClass) {
+                    } else if (to.is(BigInteger.class)) {
                         return 10;
-                    } else if (to == ScriptRuntime.ObjectClass) {
+                    } else if (to.isObjectExact()) {
                         return 11;
-                    } else if (ScriptRuntime.NumberClass.isAssignableFrom(to)) {
+                    } else if (to.isNumber()) {
                         // "double" is #1
                         return 2;
                     }
@@ -335,25 +340,25 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 break;
 
             case JSTYPE_STRING:
-                if (to == ScriptRuntime.StringClass) {
+                if (to == TypeInfo.STRING) {
                     return 1;
                 } else if (to.isInstance(fromObj)) {
                     return 2;
                 } else if (to.isPrimitive()) {
-                    if (to == Character.TYPE) {
+                    if (to.isCharacter()) {
                         return 3;
-                    } else if (to != Boolean.TYPE) {
+                    } else if (!to.isBoolean()) {
                         return 4;
                     }
                 }
                 break;
 
             case JSTYPE_JAVA_CLASS:
-                if (to == ScriptRuntime.ClassClass) {
+                if (to.is(Class.class)) {
                     return 1;
-                } else if (to == ScriptRuntime.ObjectClass) {
+                } else if (to.isObjectExact()) {
                     return 3;
-                } else if (to == ScriptRuntime.StringClass) {
+                } else if (to == TypeInfo.STRING) {
                     return 4;
                 }
                 break;
@@ -367,16 +372,16 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 if (to.isInstance(javaObj)) {
                     return CONVERSION_NONTRIVIAL;
                 }
-                if (to == ScriptRuntime.StringClass) {
+                if (to == TypeInfo.STRING) {
                     return 2;
-                } else if (to.isPrimitive() && to != Boolean.TYPE) {
+                } else if (to.isPrimitive() && !to.isBoolean()) {
                     return (fromCode == JSTYPE_JAVA_ARRAY) ? CONVERSION_NONE : 2 + getSizeRank(to);
                 }
                 break;
 
             case JSTYPE_OBJECT:
                 // Other objects takes #1-#3 spots
-                if (to != ScriptRuntime.ObjectClass && to.isInstance(fromObj)) {
+                if (!to.isObjectExact() && to.isInstance(fromObj)) {
                     // No conversion required, but don't apply for java.lang.Object
                     return 1;
                 }
@@ -387,18 +392,18 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                         // and string conversion, per LC3.
                         return 2;
                     }
-                } else if (to == ScriptRuntime.ObjectClass) {
+                } else if (to.isObjectExact()) {
                     return 3;
-                } else if (to == ScriptRuntime.StringClass) {
+                } else if (to == TypeInfo.STRING) {
                     return 4;
-                } else if (to == ScriptRuntime.DateClass) {
+                } else if (to == TypeInfo.DATE) {
                     if (fromObj instanceof NativeDate) {
                         // This is a native date to java date conversion
                         return 1;
                     }
                 } else if (to.isInterface()) {
 
-                    if (fromObj instanceof NativeFunction) {
+                    if (fromObj instanceof JSFunction) {
                         // See comments in createInterfaceAdapter
                         return 1;
                     }
@@ -406,7 +411,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                         return 2;
                     }
                     return 12;
-                } else if (to.isPrimitive() && to != Boolean.TYPE) {
+                } else if (to.isPrimitive() && !to.isBoolean()) {
                     return 4 + getSizeRank(to);
                 }
                 break;
@@ -415,6 +420,13 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
         return CONVERSION_NONE;
     }
 
+    static int getSizeRank(TypeInfo type) {
+        if (type.isPrimitive()) {
+            return getSizeRank(type.asClass());
+        }
+        return 8;
+    }
+
     static int getSizeRank(Class<?> aType) {
         if (aType == Double.TYPE) {
             return 1;
@@ -432,9 +444,8 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
             return 7;
         } else if (aType == Boolean.TYPE) {
             return CONVERSION_NONE;
-        } else {
-            return 8;
         }
+        return 8;
     }
 
     private static int getJSTypeCode(Object value) {
@@ -479,16 +490,17 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
      */
     @Deprecated
     public static Object coerceType(Class<?> type, Object value) {
-        return coerceTypeImpl(type, value);
+        return coerceTypeImpl(TypeInfoFactory.GLOBAL.create(type), value);
     }
 
     /** Type-munging for field setting and method invocation. Conforms to LC3 specification */
-    static Object coerceTypeImpl(Class<?> type, Object value) {
-        if (value != null && value.getClass() == type) {
+    static Object coerceTypeImpl(TypeInfo type, Object value) {
+        if (value != null && type.is(value.getClass())) {
             return value;
         }
 
         int jsTypeCode = getJSTypeCode(value);
+        assert (jsTypeCode == JSTYPE_NULL) == (value == null);
         switch (jsTypeCode) {
             case JSTYPE_NULL:
                 // raise error if type.isPrimitive()
@@ -498,7 +510,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 return null;
 
             case JSTYPE_UNDEFINED:
-                if (type == ScriptRuntime.StringClass || type == ScriptRuntime.ObjectClass) {
+                if (type == TypeInfo.STRING || type.isObjectExact()) {
                     return "undefined";
                 }
                 reportConversionError("undefined", type);
@@ -506,11 +518,9 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
             case JSTYPE_BOOLEAN:
                 // Under LC3, only JS Booleans can be coerced into a Boolean value
-                if (type == Boolean.TYPE
-                        || type == ScriptRuntime.BooleanClass
-                        || type == ScriptRuntime.ObjectClass) {
+                if (type.isBoolean() || type.isObjectExact()) {
                     return value;
-                } else if (type == ScriptRuntime.StringClass) {
+                } else if (type == TypeInfo.STRING) {
                     return value.toString();
                 } else {
                     reportConversionError(value, type);
@@ -519,22 +529,24 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
             case JSTYPE_NUMBER:
             case JSTYPE_BIGINT:
-                if (type == ScriptRuntime.StringClass) {
+                if (type == TypeInfo.STRING) {
                     return ScriptRuntime.toString(value);
-                } else if (type == ScriptRuntime.ObjectClass) {
+                } else if (type.isObjectExact()) {
                     Context context = Context.getCurrentContext();
                     if ((context != null)
                             && context.hasFeature(Context.FEATURE_INTEGER_WITHOUT_DECIMAL_PLACE)) {
                         // to process numbers like 2.0 as 2 without decimal place
                         long roundedValue = Math.round(toDouble(value));
                         if (roundedValue == toDouble(value)) {
-                            return coerceToNumber(Long.TYPE, value);
+                            return coerceToNumber(TypeInfo.LONG, value);
                         }
                     }
                     return coerceToNumber(
-                            jsTypeCode == JSTYPE_BIGINT ? BigInteger.class : Double.TYPE, value);
-                } else if ((type.isPrimitive() && type != Boolean.TYPE)
-                        || ScriptRuntime.NumberClass.isAssignableFrom(type)) {
+                            jsTypeCode == JSTYPE_BIGINT ? TypeInfo.BIG_INT : TypeInfo.DOUBLE,
+                            value);
+                } else if ((type.isPrimitive() && !type.isBoolean())
+                        || type.isNumber()
+                        || type.isCharacter()) {
                     return coerceToNumber(type, value);
                 } else {
                     reportConversionError(value, type);
@@ -542,9 +554,9 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 break;
 
             case JSTYPE_STRING:
-                if (type == ScriptRuntime.StringClass || type.isInstance(value)) {
+                if (type == TypeInfo.STRING || type.asClass().isInstance(value)) {
                     return value.toString();
-                } else if (type == Character.TYPE || type == ScriptRuntime.CharacterClass) {
+                } else if (type.isCharacter()) {
                     // Special case for converting a single char string to a
                     // character
                     // Placed here because it applies *only* to JS strings,
@@ -553,8 +565,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                         return Character.valueOf(((CharSequence) value).charAt(0));
                     }
                     return coerceToNumber(type, value);
-                } else if ((type.isPrimitive() && type != Boolean.TYPE)
-                        || ScriptRuntime.NumberClass.isAssignableFrom(type)) {
+                } else if ((type.isPrimitive() && !type.isBoolean()) || type.isNumber()) {
                     return coerceToNumber(type, value);
                 } else {
                     reportConversionError(value, type);
@@ -566,9 +577,9 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                     value = ((Wrapper) value).unwrap();
                 }
 
-                if (type == ScriptRuntime.ClassClass || type == ScriptRuntime.ObjectClass) {
+                if (type.is(Class.class) || type.isObjectExact()) {
                     return value;
-                } else if (type == ScriptRuntime.StringClass) {
+                } else if (type == TypeInfo.STRING) {
                     return value.toString();
                 } else {
                     reportConversionError(value, type);
@@ -581,12 +592,12 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                     value = ((Wrapper) value).unwrap();
                 }
                 if (type.isPrimitive()) {
-                    if (type == Boolean.TYPE) {
+                    if (type.isBoolean()) {
                         reportConversionError(value, type);
                     }
                     return coerceToNumber(type, value);
                 }
-                if (type == ScriptRuntime.StringClass) {
+                if (type == TypeInfo.STRING) {
                     return value.toString();
                 }
                 if (type.isInstance(value)) {
@@ -596,16 +607,16 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 break;
 
             case JSTYPE_OBJECT:
-                if (type == ScriptRuntime.StringClass) {
+                if (type == TypeInfo.STRING) {
                     return ScriptRuntime.toString(value);
                 } else if (type.isPrimitive()) {
-                    if (type == Boolean.TYPE) {
+                    if (type.isBoolean()) {
                         reportConversionError(value, type);
                     }
                     return coerceToNumber(type, value);
                 } else if (type.isInstance(value)) {
                     return value;
-                } else if (type == ScriptRuntime.DateClass && value instanceof NativeDate) {
+                } else if (type == TypeInfo.DATE && value instanceof NativeDate) {
                     double time = ((NativeDate) value).getJSTimeValue();
                     // XXX: This will replace NaN by 0
                     return new Date((long) time);
@@ -614,27 +625,28 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                     // to the target (component) type.
                     NativeArray array = (NativeArray) value;
                     long length = array.getLength();
-                    Class<?> arrayType = type.getComponentType();
-                    Object Result = Array.newInstance(arrayType, (int) length);
+                    var componentType = type.getComponentType();
+                    var result = componentType.newArray((int) length);
                     for (int i = 0; i < length; ++i) {
                         try {
-                            Array.set(Result, i, coerceTypeImpl(arrayType, array.get(i, array)));
+                            Array.set(
+                                    result, i, coerceTypeImpl(componentType, array.get(i, array)));
                         } catch (EvaluatorException ee) {
                             reportConversionError(value, type);
                         }
                     }
 
-                    return Result;
+                    return result;
                 } else if (value instanceof Wrapper) {
                     value = ((Wrapper) value).unwrap();
-                    if (type.isInstance(value)) return value;
+                    if (type.asClass().isInstance(value)) return value;
                     reportConversionError(value, type);
                 } else if (type.isInterface()
                         && (value instanceof NativeObject
                                 || (value instanceof Callable
                                         && value instanceof ScriptableObject))) {
                     // Try to use function/object as implementation of Java interface.
-                    return createInterfaceAdapter(type, (ScriptableObject) value);
+                    return createInterfaceAdapter(type.asClass(), (ScriptableObject) value);
                 } else {
                     reportConversionError(value, type);
                 }
@@ -664,11 +676,11 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
         return glue;
     }
 
-    private static Object coerceToNumber(Class<?> type, Object value) {
+    private static Object coerceToNumber(TypeInfo type, Object value) {
         Class<?> valueClass = value.getClass();
 
         // Character
-        if (type == Character.TYPE || type == ScriptRuntime.CharacterClass) {
+        if (type.isCharacter()) {
             if (valueClass == ScriptRuntime.CharacterClass) {
                 return value;
             }
@@ -682,23 +694,21 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
         }
 
         // Double, Float
-        if (type == ScriptRuntime.ObjectClass
-                || type == ScriptRuntime.DoubleClass
-                || type == Double.TYPE) {
+        if (type.isObjectExact() || type.isDouble()) {
             if (valueClass == ScriptRuntime.DoubleClass) {
                 return value;
             }
             return Double.valueOf(toDouble(value));
         }
 
-        if (type == ScriptRuntime.BigIntegerClass) {
-            if (valueClass == ScriptRuntime.BigIntegerClass) {
+        if (type.is(BigInteger.class)) {
+            if (valueClass == BigInteger.class) {
                 return value;
             }
             return ScriptRuntime.toBigInt(value);
         }
 
-        if (type == ScriptRuntime.FloatClass || type == Float.TYPE) {
+        if (type.isFloat()) {
             if (valueClass == ScriptRuntime.FloatClass) {
                 return value;
             }
@@ -719,7 +729,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
         }
 
         // Integer, Long, Short, Byte
-        if (type == ScriptRuntime.IntegerClass || type == Integer.TYPE) {
+        if (type.isInt()) {
             if (valueClass == ScriptRuntime.IntegerClass) {
                 return value;
             }
@@ -732,7 +742,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                                     Integer.MAX_VALUE));
         }
 
-        if (type == ScriptRuntime.LongClass || type == Long.TYPE) {
+        if (type.isLong()) {
             if (valueClass == ScriptRuntime.LongClass) {
                 return value;
             }
@@ -748,7 +758,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
             return Long.valueOf(toInteger(value, ScriptRuntime.LongClass, min, max));
         }
 
-        if (type == ScriptRuntime.ShortClass || type == Short.TYPE) {
+        if (type.isShort()) {
             if (valueClass == ScriptRuntime.ShortClass) {
                 return value;
             }
@@ -761,7 +771,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                                     Short.MAX_VALUE));
         }
 
-        if (type == ScriptRuntime.ByteClass || type == Byte.TYPE) {
+        if (type.isByte()) {
             if (valueClass == ScriptRuntime.ByteClass) {
                 return value;
             }
@@ -792,18 +802,13 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
             Method meth;
             try {
                 meth = value.getClass().getMethod("doubleValue", (Class[]) null);
-            } catch (NoSuchMethodException e) {
-                meth = null;
-            } catch (SecurityException e) {
+            } catch (NoSuchMethodException | SecurityException e) {
                 meth = null;
             }
             if (meth != null) {
                 try {
                     return ((Number) meth.invoke(value, (Object[]) null)).doubleValue();
-                } catch (IllegalAccessException e) {
-                    // XXX: ignore, or error message?
-                    reportConversionError(value, Double.TYPE);
-                } catch (InvocationTargetException e) {
+                } catch (IllegalAccessException | InvocationTargetException e) {
                     // XXX: ignore, or error message?
                     reportConversionError(value, Double.TYPE);
                 }
@@ -843,6 +848,14 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
                 JavaMembers.javaSignature(type));
     }
 
+    @SuppressWarnings("DoNotCallSuggester")
+    static void reportConversionError(Object value, TypeInfo type) {
+        // It uses String.valueOf(value), not value.toString() since
+        // value can be null, bug 282447.
+        throw Context.reportRuntimeErrorById(
+                "msg.conversion.not.allowed", String.valueOf(value), type);
+    }
+
     private void writeObject(ObjectOutputStream out) throws IOException {
         out.defaultWriteObject();
 
@@ -861,8 +874,8 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
             out.writeObject(javaObject);
         }
 
-        if (staticType != null) {
-            out.writeObject(staticType.getName());
+        if (staticType != TypeInfo.NONE) {
+            out.writeObject(staticType.asClass().getName());
         } else {
             out.writeObject(null);
         }
@@ -886,9 +899,9 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
         String className = (String) in.readObject();
         if (className != null) {
-            staticType = Class.forName(className);
+            staticType = TypeInfoFactory.GLOBAL.create(Class.forName(className));
         } else {
-            staticType = null;
+            staticType = TypeInfo.NONE;
         }
 
         initMembers();
@@ -959,7 +972,7 @@ public class NativeJavaObject implements Scriptable, SymbolScriptable, Wrapper,
 
     protected transient Object javaObject;
 
-    protected transient Class<?> staticType;
+    protected transient TypeInfo staticType;
     protected transient JavaMembers members;
     private transient Map<String, FieldAndMethods> fieldAndMethods;
     protected transient boolean isAdapter;
