@@ -68,7 +68,9 @@ public class Node implements Iterable<Node> {
             OBJECT_LITERAL_DESTRUCTURING = 29,
             OPTIONAL_CHAINING = 30,
             SUPER_PROPERTY_ACCESS = 31,
-            LAST_PROP = SUPER_PROPERTY_ACCESS;
+            NUMBER_OF_SPREAD = 32,
+            LAST_PROP = NUMBER_OF_SPREAD,
+            FIRST_PROP = FUNCTION_PROP;
 
     // values of ISNUMBER_PROP to specify
     // which of the children are Number types
@@ -378,75 +380,88 @@ public class Node implements Iterable<Node> {
         return new NodeIterator();
     }
 
-    private static final String propToString(int propType) {
-        if (Token.printTrees) {
-            // If Context.printTrees is false, the compiler
-            // can remove all these strings.
-            switch (propType) {
-                case FUNCTION_PROP:
-                    return "function";
-                case LOCAL_PROP:
-                    return "local";
-                case LOCAL_BLOCK_PROP:
-                    return "local_block";
-                case REGEXP_PROP:
-                    return "regexp";
-                case CASEARRAY_PROP:
-                    return "casearray";
+    private static String propToString(int propType) {
+        if (!Token.printTrees) {
+            return null;
+        }
+        // If Context.printTrees is false, the compiler
+        // can remove all these strings.
+        return propName(propType);
+    }
 
-                case TARGETBLOCK_PROP:
-                    return "targetblock";
-                case VARIABLE_PROP:
-                    return "variable";
-                case ISNUMBER_PROP:
-                    return "isnumber";
-                case DIRECTCALL_PROP:
-                    return "directcall";
+    static String propName(int propType) {
+        switch (propType) {
+            case FUNCTION_PROP:
+                return "function";
+            case LOCAL_PROP:
+                return "local";
+            case LOCAL_BLOCK_PROP:
+                return "local_block";
+            case REGEXP_PROP:
+                return "regexp";
+            case CASEARRAY_PROP:
+                return "casearray";
 
-                case SPECIALCALL_PROP:
-                    return "specialcall";
-                case SKIP_INDEXES_PROP:
-                    return "skip_indexes";
-                case OBJECT_IDS_PROP:
-                    return "object_ids_prop";
-                case INCRDECR_PROP:
-                    return "incrdecr_prop";
-                case CATCH_SCOPE_PROP:
-                    return "catch_scope_prop";
-                case LABEL_ID_PROP:
-                    return "label_id_prop";
-                case MEMBER_TYPE_PROP:
-                    return "member_type_prop";
-                case NAME_PROP:
-                    return "name_prop";
-                case CONTROL_BLOCK_PROP:
-                    return "control_block_prop";
-                case PARENTHESIZED_PROP:
-                    return "parenthesized_prop";
-                case GENERATOR_END_PROP:
-                    return "generator_end";
-                case DESTRUCTURING_ARRAY_LENGTH:
-                    return "destructuring_array_length";
-                case DESTRUCTURING_NAMES:
-                    return "destructuring_names";
-                case DESTRUCTURING_PARAMS:
-                    return "destructuring_params";
-                case EXPRESSION_CLOSURE_PROP:
-                    return "expression_closure_prop";
-                case TEMPLATE_LITERAL_PROP:
-                    return "template_literal";
-                case TRAILING_COMMA:
-                    return "trailing comma";
-                case OPTIONAL_CHAINING:
-                    return "optional_chaining";
-                case SUPER_PROPERTY_ACCESS:
-                    return "super_property_access";
+            case TARGETBLOCK_PROP:
+                return "targetblock";
+            case VARIABLE_PROP:
+                return "variable";
+            case ISNUMBER_PROP:
+                return "isnumber";
+            case DIRECTCALL_PROP:
+                return "directcall";
 
-                default:
-                    Kit.codeBug();
-            }
+            case SPECIALCALL_PROP:
+                return "specialcall";
+            case SKIP_INDEXES_PROP:
+                return "skip_indexes";
+            case OBJECT_IDS_PROP:
+                return "object_ids_prop";
+            case INCRDECR_PROP:
+                return "incrdecr_prop";
+            case CATCH_SCOPE_PROP:
+                return "catch_scope_prop";
+            case LABEL_ID_PROP:
+                return "label_id_prop";
+            case MEMBER_TYPE_PROP:
+                return "member_type_prop";
+            case NAME_PROP:
+                return "name_prop";
+            case CONTROL_BLOCK_PROP:
+                return "control_block_prop";
+            case PARENTHESIZED_PROP:
+                return "parenthesized_prop";
+            case GENERATOR_END_PROP:
+                return "generator_end";
+            case DESTRUCTURING_ARRAY_LENGTH:
+                return "destructuring_array_length";
+            case DESTRUCTURING_NAMES:
+                return "destructuring_names";
+            case DESTRUCTURING_PARAMS:
+                return "destructuring_params";
+            case JSDOC_PROP:
+                return "jsdoc";
+            case EXPRESSION_CLOSURE_PROP:
+                return "expression_closure_prop";
+            case ARROW_FUNCTION_PROP:
+                return "arrow_function";
+            case TEMPLATE_LITERAL_PROP:
+                return "template_literal";
+            case TRAILING_COMMA:
+                return "trailing comma";
+            case OBJECT_LITERAL_DESTRUCTURING:
+                return "object_literal_destructuring";
+            case OPTIONAL_CHAINING:
+                return "optional_chaining";
+            case SUPER_PROPERTY_ACCESS:
+                return "super_property_access";
+            case NUMBER_OF_SPREAD:
+                return "number_of_spread";
+
+            default:
+                Kit.codeBug();
+                return null;
         }
-        return null;
     }
 
     private PropListItem lookupProperty(int propType) {
@@ -539,7 +554,7 @@ public class Node implements Iterable<Node> {
         this.column = column;
     }
 
-    /** Can only be called when <code>getType() == Token.NUMBER</code> */
+    /** Can only be called when {@code getType() == Token.NUMBER} */
     public final double getDouble() {
         return ((NumberLiteral) this).getNumber();
     }
@@ -548,7 +563,7 @@ public class Node implements Iterable<Node> {
         ((NumberLiteral) this).setNumber(number);
     }
 
-    /** Can only be called when <code>getType() == Token.BIGINT</code> */
+    /** Can only be called when {@code getType() == Token.BIGINT} */
     public BigInteger getBigInt() {
         throw new UnsupportedOperationException("Can only be called when Token.BIGINT");
     }
@@ -1181,7 +1196,7 @@ public class Node implements Iterable<Node> {
                             Object[] a = (Object[]) x.objectValue;
                             sb.append("[");
                             for (int i = 0; i < a.length; i++) {
-                                sb.append(a[i].toString());
+                                if (a[i] != null) sb.append(a[i].toString());
                                 if (i + 1 < a.length) sb.append(", ");
                             }
                             sb.append("]");
