@@ -5,11 +5,11 @@
 package org.mozilla.javascript;
 
 /**
- * A <code>java.lang.SecurityManager</code> subclass that provides access to the current top-most
- * script class on the execution stack. This can be used to get the class loader or protection
- * domain of the script that triggered the current action. It is required for JavaAdapters to have
- * the same <code>ProtectionDomain</code> as the script code that created them. Embeddings that
- * implement their own SecurityManager can use this as base class.
+ * A {@code java.lang.SecurityManager} subclass that provides access to the current top-most script
+ * class on the execution stack. This can be used to get the class loader or protection domain of
+ * the script that triggered the current action. It is required for JavaAdapters to have the same
+ * {@code ProtectionDomain} as the script code that created them. Embeddings that implement their
+ * own SecurityManager can use this as base class.
  */
 public class RhinoSecurityManager extends SecurityManager {
 
@@ -22,7 +22,7 @@ public class RhinoSecurityManager extends SecurityManager {
     protected Class<?> getCurrentScriptClass() {
         Class<?>[] context = getClassContext();
         for (Class<?> c : context) {
-            if (((c != InterpretedFunction.class) && NativeFunction.class.isAssignableFrom(c))
+            if (((c != JSFunction.class) && (c != JSScript.class))
                     || PolicySecurityController.SecureCaller.class.isAssignableFrom(c)) {
                 return c;
             }
