@@ -9,6 +9,7 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 
 import java.io.IOException;
 import java.util.List;
@@ -22,6 +23,7 @@ import org.mozilla.javascript.EvaluatorException;
 import org.mozilla.javascript.Node;
 import org.mozilla.javascript.Parser;
 import org.mozilla.javascript.Token;
+import org.mozilla.javascript.ast.AbstractObjectProperty;
 import org.mozilla.javascript.ast.Assignment;
 import org.mozilla.javascript.ast.AstNode;
 import org.mozilla.javascript.ast.AstRoot;
@@ -554,7 +556,7 @@ public class ParserTest {
         List<CatchClause> catchBlocks = tryStmt.getCatchClauses();
         CatchClause catchClause = catchBlocks.get(0);
         Scope catchVarBlock = catchClause.getBody();
-        Name catchVar = catchClause.getVarName();
+        Name catchVar = assertInstanceOf(Name.class, catchClause.getVarName());
         AstNode finallyBlock = tryStmt.getFinallyBlock();
         AstNode finallyStmt = (AstNode) finallyBlock.getFirstChild();
 
@@ -726,17 +728,20 @@ public class ParserTest {
         Name firstVarName = (Name) firstInitializer.getTarget();
 
         ObjectLiteral objectLiteral = (ObjectLiteral) firstInitializer.getInitializer();
-        List<ObjectProperty> props = objectLiteral.getElements();
-        ObjectProperty firstObjectLit = props.get(0);
-        ObjectProperty secondObjectLit = props.get(1);
-        ObjectProperty thirdObjectLit = props.get(2);
+        List<AbstractObjectProperty> props = objectLiteral.getElements();
+        assertTrue(props.get(0) instanceof ObjectProperty);
+        var firstObjectLit = props.get(0);
+        assertTrue(props.get(1) instanceof ObjectProperty);
+        var secondObjectLit = props.get(1);
+        assertTrue(props.get(2) instanceof ObjectProperty);
+        var thirdObjectLit = props.get(2);
 
-        AstNode firstKey = firstObjectLit.getLeft();
-        AstNode firstValue = firstObjectLit.getRight();
-        AstNode secondKey = secondObjectLit.getLeft();
-        AstNode secondValue = secondObjectLit.getRight();
-        AstNode thirdKey = thirdObjectLit.getLeft();
-        AstNode thirdValue = thirdObjectLit.getRight();
+        AstNode firstKey = ((ObjectProperty) firstObjectLit).getKey();
+        AstNode firstValue = ((ObjectProperty) firstObjectLit).getValue();
+        AstNode secondKey = ((ObjectProperty) secondObjectLit).getKey();
+        AstNode secondValue = ((ObjectProperty) secondObjectLit).getValue();
+        AstNode thirdKey = ((ObjectProperty) thirdObjectLit).getKey();
+        AstNode thirdValue = ((ObjectProperty) thirdObjectLit).getValue();
 
         assertLineColumnAre(1, 5, firstVarName);
         assertLineColumnAre(2, 1, objectLiteral);
@@ -982,7 +987,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        NumberLiteral number = (NumberLiteral) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        NumberLiteral number = (NumberLiteral) prop.getKey();
         assertNotNull(number.getJsDoc());
     }
 
@@ -994,9 +1000,10 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        for (ObjectProperty el : lit.getElements()) {
-            assertNull(el.getLeft().getJsDoc());
-            assertNull(el.getRight().getJsDoc());
+        for (AbstractObjectProperty abstractEl : lit.getElements()) {
+            ObjectProperty el = (ObjectProperty) abstractEl;
+            assertNull(el.getKey().getJsDoc());
+            assertNull(el.getValue().getJsDoc());
         }
     }
 
@@ -1008,7 +1015,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        StringLiteral stringLit = (StringLiteral) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        StringLiteral stringLit = (StringLiteral) prop.getKey();
         assertNotNull(stringLit.getJsDoc());
     }
 
@@ -1020,8 +1028,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        ParenthesizedExpression parens =
-                (ParenthesizedExpression) lit.getElements().get(0).getRight();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        ParenthesizedExpression parens = (ParenthesizedExpression) prop.getValue();
         assertNotNull(parens.getJsDoc());
     }
 
@@ -1033,7 +1041,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        Name objLitKey = (Name) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        Name objLitKey = (Name) prop.getKey();
         assertNotNull(objLitKey.getJsDoc());
     }
 
@@ -1045,8 +1054,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        ParenthesizedExpression parens =
-                (ParenthesizedExpression) lit.getElements().get(0).getRight();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        ParenthesizedExpression parens = (ParenthesizedExpression) prop.getValue();
         assertNotNull(parens.getJsDoc());
     }
 
@@ -1058,7 +1067,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        Name objLitKey = (Name) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        Name objLitKey = (Name) prop.getKey();
         assertNotNull(objLitKey.getJsDoc());
     }
 
@@ -1070,7 +1080,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        NumberLiteral number = (NumberLiteral) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        NumberLiteral number = (NumberLiteral) prop.getKey();
         assertNotNull(number.getJsDoc());
     }
 
@@ -1082,7 +1093,8 @@ public class ParserTest {
         ExpressionStatement st = (ExpressionStatement) root.getFirstChild();
         ParenthesizedExpression pt = (ParenthesizedExpression) st.getExpression();
         ObjectLiteral lit = (ObjectLiteral) pt.getExpression();
-        StringLiteral stringLit = (StringLiteral) lit.getElements().get(0).getLeft();
+        ObjectProperty prop = (ObjectProperty) lit.getElements().get(0);
+        StringLiteral stringLit = (StringLiteral) prop.getKey();
         assertNotNull(stringLit.getJsDoc());
     }
 
@@ -1455,16 +1467,16 @@ public class ParserTest {
         assertTrue(((Assignment) expr.getExpression()).getRight() instanceof ObjectLiteral);
         ObjectLiteral obj = (ObjectLiteral) ((Assignment) expr.getExpression()).getRight();
         assertEquals(1, obj.getElements().size());
-        ObjectProperty g = obj.getElements().get(0);
+        ObjectProperty g = (ObjectProperty) obj.getElements().get(0);
 
-        assertTrue(g.getLeft() instanceof GeneratorMethodDefinition);
-        assertLineColumnAre(0, 7, g.getLeft());
-        AstNode genMethodName = ((GeneratorMethodDefinition) g.getLeft()).getMethodName();
+        assertTrue(g.getKey() instanceof GeneratorMethodDefinition);
+        assertLineColumnAre(0, 7, g.getKey());
+        AstNode genMethodName = ((GeneratorMethodDefinition) g.getKey()).getMethodName();
         assertTrue(genMethodName instanceof Name);
         assertLineColumnAre(0, 8, genMethodName);
 
-        assertTrue(g.getRight() instanceof FunctionNode);
-        assertTrue(((FunctionNode) g.getRight()).isES6Generator());
+        assertTrue(g.getValue() instanceof FunctionNode);
+        assertTrue(((FunctionNode) g.getValue()).isES6Generator());
     }
 
     @Test
@@ -1479,6 +1491,25 @@ public class ParserTest {
         expectParseErrors("`\\u{8", new String[] {"syntax error"});
     }
 
+    @Test
+    public void errorOnInvalidDestructuringDeclaration() {
+        expectParseErrors(
+                "for(var {};;) {}",
+                new String[] {"Missing = in destructuring declaration", "syntax error"});
+    }
+
+    @Test
+    public void commentValueIsNotTruncatedBecauseOfEof() {
+        String source = "// comment";
+
+        AstRoot root = parse(source);
+        assertNotNull(root.getComments());
+        assertEquals(1, root.getComments().size());
+        Comment comment = root.getComments().first();
+        assertEquals(source, comment.getValue());
+        assertEquals(10, comment.getLength());
+    }
+
     private void expectParseErrors(String string, String[] errors) {
         parse(string, errors, null, false);
     }
