@@ -1,7 +1,7 @@
 package org.mozilla.javascript.tests.es5;
 
 import org.junit.Test;
-import org.mozilla.javascript.tests.Utils;
+import org.mozilla.javascript.testutils.Utils;
 
 public class GeneratorToStringTest {
     @Test
