@@ -13,8 +13,16 @@ load("testsrc/assert.js");
   var a = new Float64Array(4);
   assertEquals('Float64Array', a[Symbol.toStringTag]);
   assertEquals(false, a.hasOwnProperty(Symbol.toStringTag));
+
+  assertEquals(undefined, Float64Array[Symbol.toStringTag]);
   assertEquals(false, Float64Array.hasOwnProperty(Symbol.toStringTag));
   assertEquals(false, Float64Array.prototype.hasOwnProperty(Symbol.toStringTag));
 })();
 
+(function TestPrototypeSymbolToString() {
+  var a = new Float64Array(4).__proto__;
+  assertEquals(undefined, a[Symbol.toStringTag]);
+  assertEquals(false, a.hasOwnProperty(Symbol.toStringTag));
+})();
+
 "success";
