@@ -12,7 +12,9 @@ obj = {
   }
 };
 assertEquals(123, obj.a());
-// assertEquals("a", obj.a.name);
+assertEquals("a", obj.a.name);
+assertEquals(undefined, obj.a.prototype);
+assertEquals(undefined, Object.getOwnPropertyDescriptor(obj.a, 'prototype'));
 
 assertEquals("abcefg", {
   abc() {
@@ -63,7 +65,10 @@ assertEquals(123, {
 }.abc());
 
 // Method is the kind of function, that is non-constructor.
-// assertThrows('new (({ a() {} }).a)', TypeError);
+assertThrows(function() {
+  new (({ a() {} }).a)
+}, TypeError);
+assertNotNull(new (({ notAMethod: function() {} }).notAMethod));
 
 var desc = Object.getOwnPropertyDescriptor({
   a() {
