@@ -20,7 +20,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Int8Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int8Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
@@ -28,7 +28,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Int8Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int8Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -87,7 +87,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint8Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint8Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
@@ -95,7 +95,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint8Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint8Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -155,15 +155,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Int16Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int16Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 2", msg);
 
   msg = null;
   try { new Int16Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -2 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int16Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -225,15 +225,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint16Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint16Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 2", msg);
 
   msg = null;
   try { new Uint16Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -2 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint16Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -295,15 +295,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Int32Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int32Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 4", msg);
 
   msg = null;
   try { new Int32Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -4 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Int32Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -365,15 +365,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint32Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint32Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 4", msg);
 
   msg = null;
   try { new Uint32Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -4 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint32Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -435,7 +435,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint8ClampedArray(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint8ClampedArray(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
@@ -443,7 +443,7 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Uint8ClampedArray(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Uint8ClampedArray(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -503,15 +503,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Float32Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Float32Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 4", msg);
 
   msg = null;
   try { new Float32Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -4 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Float32Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
@@ -573,15 +573,15 @@ load("testsrc/assert.js");
 
   msg = null;
   try { new Float64Array(new ArrayBuffer(), -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset -1 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Float64Array(new ArrayBuffer(2), 3); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: offset 3 out of range", msg);
+  assertEquals("RangeError: offset 3 must be a multiple of the byte size 8", msg);
 
   msg = null;
   try { new Float64Array(new ArrayBuffer(), 0, -1); } catch (e) { msg = e.toString();}
-  assertEquals("RangeError: length -8 out of range", msg);
+  assertEquals("RangeError: Index -1 out of range", msg);
 
   msg = null;
   try { new Float64Array(new ArrayBuffer(2), 0, 3); } catch (e) { msg = e.toString();}
