@@ -4,10 +4,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = '9.1.1.5.js';
+gTestfile = '9.1.1.11.js';
 
-// XML Type [[Default Value]]
-START("9.1.1.5 - XML [[Default Value]]");
+START("9.1.1.11 - XML [[Insert]]");
 
 var actual = 'Test not implemented';
 var expect = 'PASS';
