@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-gTestfile = '7.4.3-14-n.js';
+gTestfile = '7.4.3-8-n.js';
 
 /**
-   File Name:          7.4.3-14-n.js
+   File Name:          7.4.3-8-n.js
    ECMA Section:       7.4.3
 
    Description:
@@ -23,48 +23,16 @@ gTestfile = '7.4.3-14-n.js';
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
-var SECTION = "7.4.3-14-n";
+var SECTION = "7.4.3-9-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Future Reserved Words";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-var actual = 'no error';
-var prefValue;
-
-print("This test requires option javascript.options.strict enabled");
-
-if (!options().match(/strict/))
-{
-  options('strict');
-}
-if (!options().match(/werror/))
-{
-  options('werror');
-}
-
-try
-{
-  eval("var enum = true");
-}
-catch(e)
-{
-  actual = 'error';
-}
-
-DESCRIPTION = "var enum = true";
+DESCRIPTION = "var switch = true";
 EXPECTED = "error";
 
-// force exception since this is a negative test
-if (actual == 'error')
-{
-  throw actual;
-}
-
-new TestCase( SECTION, 
-              "var enum = true",    
-              "error",   
-              actual );
+new TestCase( SECTION,  "var switch = true",     "error",    eval("var switch = true") );
 
 test();
