@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-457065-03.js';
+var gTestfile = 'regress-460886-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 457065;
-var summary = 'Do not assert: !fp->callee || fp->thisp == JSVAL_TO_OBJECT(fp->argv[-1])';
+var BUGNUMBER = 460886;
+var summary = 'Do not crash @ js_NewStringCopy';
 var actual = '';
 var expect = '';
 
@@ -21,14 +21,8 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  jit(true);
-
-  (function() {
-    new function (){ for (var x = 0; x < 3; ++x){} };
-  })();
-
-  jit(false);
-
+  for (var j = 0; j < 5; ++j) { "".substring(-60000); } 
+ 
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
