@@ -3,14 +3,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-435497-02.js';
+var gTestfile = 'regress-455464-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 435497;
-var summary = 'Do not assert op2 == JSOP_INITELEM';
+var BUGNUMBER = 455464;
+var summary = 'Do not assert with JIT: !TRACE_RECORDER(cx) ^ (jumpTable == recordingJumpTable)';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-
 //-----------------------------------------------------------------------------
 test();
 //-----------------------------------------------------------------------------
@@ -20,14 +19,12 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  try
-  {
-    eval('(function() { x setter = 0, y; const x; })();');
-  }
-  catch(ex)
-  {
-  }
+
+  jit(true);
+
+  a=b=c=d=0; this.__defineGetter__('g', gc); for each (y in this);
+
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
