@@ -6,7 +6,7 @@
 var gTestfile = 'regress-352022.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 352022;
-var summary = 'decompilation old, bad bug dropping parenthesis';
+var summary = 'decompilation of let, delete and parens';
 var actual = '';
 var expect = '';
 
@@ -20,12 +20,18 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  var f;
 
-  var f = function (){a[b] = (c, d)}
+  f = function() { g(h) = (delete let (y) 3); }
   actual = f + '';
-  expect = 'function () {\n    a[b] = (c, d)\n}';
+  expect = 'function () {\n    g(h) = ((let (y) 3), true);\n}';
+  compareSource(expect, actual, summary + ': 1');
 
-  compareSource(expect, actual, summary);
+  f = function () {    g(h) = ((let (y) 3), true);}
+  actual = f + '';
+  expect = 'function () {\n    g(h) = ((let (y) 3), true);\n}';
+  compareSource(expect, actual, summary + ': 2');
 
   exitFunc ('test');
 }
