@@ -3,12 +3,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-349012-05.js';
+var gTestfile = 'regress-345879-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 349012;
-var summary = 'generator recursively calling itself via close is an Error';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 345879;
+var summary = 'Crash when calling a function from a generator with less arguments than its arity ';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
 //-----------------------------------------------------------------------------
@@ -20,26 +20,15 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  var iter;
+ 
   function gen() {
-    iter.close();
-    yield 1;
+    yield isNaN();
   }
 
-  expect = /TypeError.*[aA]lready executing generator/;
-  try
-  {
-    iter = gen();
-    var i = iter.next();
-    print("i="+i);
-  }
-  catch(ex)
-  {
-    print(ex + '');
-    actual = ex + '';
-  }
-  reportMatch(expect, actual, summary);
+  f = gen();
+  f.next();
+
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
