@@ -3,41 +3,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var gTestfile = 'regress-467495-04.js';
+var gTestfile = 'regress-407727-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 467495;
-var summary = 'TCF_FUN_CLOSURE_VS_VAR is necessary';
+var BUGNUMBER = 407727;
+var summary = 'let Object block redeclaration';
 var actual = '';
-var expect = '';
+var expect = 1;
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
-
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-  function g() {
-    if (1)
-      function x() {};
-    var x;
-    const y = 1;
-  }
-
-  try
-  {
-    g();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
-
+{ 
+  let Object = 1;
+  actual = Object;
   reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
 }
